﻿using System;
using GameUtils.Graphics.Data;
using SharpDX;
using SharpDX.Direct2D1;
using Vector2 = GameUtils.Math.Vector2;

namespace GameUtils.Renderers.Direct2D1_1
{
    internal class Direct2DRadialGradientBrush : RadialGradientBrushData, IDirect2DBrush
    {
        RadialGradientBrush brush;
        Graphics.GradientStop[] stops;
        Matrix3x2 matrix;
        readonly RenderTarget target;

        public Brush Brush
        {
            get { return brush; }
        }

        protected override Graphics.GradientStop[] GradientStops
        {
            get { return stops; }
            set
            {
                if (value == null)
                    throw new ArgumentNullException("value");
                if (value.Length < 2)
                    throw new ArgumentException("Es müssen mindestens zwei Verlaufsfarben angegeben sein.", "value");

                stops = value;
                var props = new RadialGradientBrushProperties();
                props.Center = brush.Center;
                props.RadiusX = brush.RadiusX;
                props.RadiusY = brush.RadiusY;
                brush.Dispose();
                brush = new RadialGradientBrush(target, props, new GradientStopCollection(target, Direct2DHelper.ConvertStops(stops)));
                brush.Transform = matrix;
            }
        }

        protected override Vector2 Center
        {
            get { return Direct2DHelper.PointFToVector2(brush.Center); }
            set { brush.Center = Direct2DHelper.Vector2ToPointF(value); }
        }

        protected override float RadiusX
        {
            get { return brush.RadiusX; }
            set { brush.RadiusX = value; }
        }

        protected override float RadiusY
        {
            get { return brush.RadiusY; }
            set { brush.RadiusY = value; }
        }

        protected override Vector2 OriginOffset
        {
            get { return Direct2DHelper.PointFToVector2(brush.GradientOriginOffset); }
            set { brush.GradientOriginOffset = Direct2DHelper.Vector2ToPointF(value); }
        }

        protected override float Opacity
        {
            get { return brush.Opacity; }
            set { brush.Opacity = value; }
        }

        internal Direct2DRadialGradientBrush(RenderTarget target, Graphics.GradientStop[] stops, Vector2 center, float radiusX, float radiusY)
        {
            matrix = Matrix3x2.Identity;
            this.target = target;
            this.stops = stops;
            var props = new RadialGradientBrushProperties();
            props.Center = Direct2DHelper.Vector2ToPointF(center);
            props.RadiusX = radiusX;
            props.RadiusY = radiusY;
            brush = new RadialGradientBrush(target, props, new GradientStopCollection(target, Direct2DHelper.ConvertStops(stops)));
        }

        protected override void SetTransform(Math.Matrix2x3 matrix)
        {
            var d2dMatrix = new Matrix3x2();
            d2dMatrix.TranslationVector = new SharpDX.Vector2(matrix.OffsetX, matrix.OffsetY);
            d2dMatrix.ScaleVector = new SharpDX.Vector2(matrix[0, 0], matrix[1, 1]);
            d2dMatrix.M12 = matrix[1, 0];
            d2dMatrix.M21 = matrix[0, 1];
            brush.Transform = d2dMatrix;
            this.matrix = d2dMatrix;
        }

        protected override void ResetTransform()
        {
            brush.Transform = Matrix3x2.Identity;
            matrix = Matrix3x2.Identity;
        }

        protected override void Dispose(bool disposing)
        {
            brush.Dispose();
        }
    }
}
