﻿using System;
using System.Threading;
using System.Windows.Forms;
using GameUtils;
using GameUtils.Graphics;
using GameUtils.Logging;
using GameUtils.Renderers.Gdi;
using GameUtils.Renderers.Direct2D1;
using GameUtils.Renderers.Direct2D1_1;
using GameUtils.Input.DefaultDevices;

namespace TestApp
{
    static class Program
    {
        /// <summary>
        /// Der Haupteinstiegspunkt für die Anwendung.
        /// </summary>
        [STAThread]
        static void Main()
        {
            Application.EnableVisualStyles();
            Application.SetCompatibleTextRenderingDefault(false);
            
            var logFile = new LogFile(@".\log.txt", false, LogMessagePriority.Engine);
            GameEngine.RegisterComponent(logFile);

            var window = new GameWindow();
            var renderers = new Renderer[] { new GdiRenderer(), new Direct2D1Renderer(), new Direct2D1_1Renderer() };
            var renderer = Renderer.IntelligentSelect(renderers, window);
            //var renderer = Renderer.Create<GdiRenderer>(window);
            GameEngine.RegisterComponent(renderer);
            var loop = new GameLoop();
            GameEngine.RegisterComponent(loop);
            var intro = new EngineIntro();
            var introHandle = loop.Register(intro);
            intro.AnimationComplete += (sender, e) =>
            {
                introHandle.Dispose();
                loop.Register(new GeometryTester());
            };
            var keyboard = new Keyboard();
            GameEngine.RegisterComponent(keyboard);
            keyboard.Initialize(window);
            keyboard.RegisterListener(window);

            keyboard.BeginCapture();
            loop.Start();

            window.Closing += (sender, e) =>
            {
                logFile.Close();
                keyboard.EndCapture();
                loop.Stop();
            };
            Application.Run(window);
        }
    }
}
